// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>NameRelationshipEditorInput class.</p>
 *
 * @author n.hoffmann
 * @created Oct 19, 2010
 * @version 1.0
 */
public class NameRelationshipEditorInput extends AbstractBulkEditorInput<NameRelationship> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7453923697752787687L;

	/** Constant <code>ID="bulkeditor.input.nameRelationship"</code> */
	public static final String ID = "bulkeditor.input.nameRelationship";

	private static NameRelationshipEditorInput instance;

	/**
	 * <p>Getter for the field <code>instance</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	public static AbstractBulkEditorInput<NameRelationship> getInstance(){
		if(instance == null){
			instance = new NameRelationshipEditorInput();
		}
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	/** {@inheritDoc} */
	@Override
	public String getName() {
		return BulkEditorInputType.NAME_RELATIONSHIP.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	/** {@inheritDoc} */
	@Override
	public String getToolTipText() {
		return getName();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#save(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public boolean save(NameRelationship entity) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#delete(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public boolean delete(NameRelationship entity) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#listEntities(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
	 */
	/** {@inheritDoc} */
	@Override
	protected List<NameRelationship> listEntities(
			IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.getSearchManager().findNameRelationships(configurator);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#loadEntity(java.util.UUID)
	 */
	/** {@inheritDoc} */
	@Override
	protected NameRelationship loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{}); 
		return null; // CdmStore.getNameService().load(entityUuid, propertyPaths);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#createEntityCreator()
	 */
	@Override
	protected IEntityCreator<NameRelationship> createEntityCreator() {
		MessagingUtils.notImplementedMessage(getClass());
		return null;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getTypeText(java.lang.Object)
	 */
	@Override
	public String getTypeText(Object entity) {
		if(entity instanceof NameRelationship){
			return ((NameRelationship) entity).getType().getTitleCache();
		}
		
		return super.getTypeText(entity);
	}

}
