// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator;

import java.util.HashMap;
import java.util.Map;

import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.reference.ReferenceFactory;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * <p>ReferenceCreator class.</p>
 *
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class ReferenceCreator implements IEntityCreator<Reference> {

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityCreator#createEntity(java.lang.String)
	 */
	/** {@inheritDoc} */
	public Reference createEntity(String text) {
		return createEntity(Reference.class, text);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator#createEntity(java.lang.Class, java.lang.String)
	 */
	/** {@inheritDoc} */
	public Reference createEntity(Object key, String text) {
		Reference reference = null;
		
		// TODO replace w more generic method when cdmlib matures accordingly
		if  (ReferenceType.Article.getMessage().equals(key)) {
			reference = ReferenceFactory.newArticle();
		}
		if  (ReferenceType.Book.getMessage().equals(key)) {
			reference = ReferenceFactory.newBook();
		}
		if  (ReferenceType.BookSection.getMessage().equals(key)) {
			reference = ReferenceFactory.newBookSection();
		}
		if  (ReferenceType.CdDvd.getMessage().equals(key)) {
			reference = ReferenceFactory.newCdDvd();
		}
		if  (ReferenceType.Database.getMessage().equals(key)) {
			reference = ReferenceFactory.newDatabase();
		}
		if  (ReferenceType.Generic.getMessage().equals(key)) {
			reference = ReferenceFactory.newGeneric();
		}
		if  (ReferenceType.InProceedings.getMessage().equals(key)) {
			reference = ReferenceFactory.newInProceedings();
		}
		if  (ReferenceType.Journal.getMessage().equals(key)) {
			reference = ReferenceFactory.newJournal();
		}
		if  (ReferenceType.Map.getMessage().equals(key)) {
			reference = ReferenceFactory.newMap();
		}
		if  (ReferenceType.Patent.getMessage().equals(key)) {
			reference = ReferenceFactory.newPatent();
		}
		if  (ReferenceType.PersonalCommunication.getMessage().equals(key)) {
			reference = ReferenceFactory.newPersonalCommunication();
		}
		if  (ReferenceType.PrintSeries.getMessage().equals(key)) {
			reference = ReferenceFactory.newPrintSeries();
		}
		if  (ReferenceType.Proceedings.getMessage().equals(key)) {
			reference = ReferenceFactory.newProceedings();
		}
		if  (ReferenceType.Report.getMessage().equals(key)) {
			reference = ReferenceFactory.newReport();
		}
		if  (ReferenceType.Thesis.getMessage().equals(key)) {
			reference = ReferenceFactory.newThesis();
		}
		if  (ReferenceType.WebPage.getMessage().equals(key)) {
			reference = ReferenceFactory.newWebPage();
		}
		if (reference == null) {
			MessagingUtils.warn(getClass(), "Reference type " + key + " not found. Creating reference with default type.");
			reference = ReferenceFactory.newGeneric();
		}
		reference.setTitleCache(text, true);
		return reference;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator#getClassLabelPairs()
	 */
	/**
	 * <p>getKeyLabelPairs</p>
	 *
	 * @return a {@link java.util.Map} object.
	 */
	public Map<Object, String> getKeyLabelPairs() {
		Map<Object, String> result = new HashMap<Object, String>();
		for (ReferenceType type : ReferenceType.values()) {
			result.put(type.getMessage(), type.getMessage());
		}
		return result;
	}

	@Override
	public boolean savesEntity() {
		// TODO Auto-generated method stub
		return false;
	}
}
